#include "histogram.hpp"
#include <eosim/core/model.hpp>
#include <cmath>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>

using namespace eosim::statics;
using namespace std;


const int largoLinea = 80;
const int largoDouble = 8;
const int largoAster = largoLinea - 2*(largoDouble + 2) - 1;

Histogram::Histogram(double lower_, double upper_,double width_, string name_):  lower(lower_), upper(upper_),
                                                                  width(width_), count(0), total(0), sosq(0),
                                                                  minX(0), maxX(0), minY(0), maxY(0), cantCell(0), name(name_),
                                                                  model(0) {

    assert(lower < upper && width > 0 && width < upper - lower && "Histogram::Histogram: Error creating a Histogram");
    cantCell = (int) ceil ((upper - lower)/width) + 2;
    for (int i = 0; i < cantCell; i++) {
        cell.push_back(0);
    }
}

void Histogram::log(double x_, double y_) {
    assert(model != 0 && "Histogram::log: Error logging data to a Histogram");
    int i = (x_ < lower) ? 0 : (x_ > upper) ? cantCell - 1 : ((int) ceil ((x_-lower)/width)) + 1;
    cell[i] += y_;
    maxX = x_ > maxX ? x_ : maxX;
    minX = x_ < minX ? x_ : minX;
    maxY = cell[i] > maxY ? cell[i] : maxY;
    minY = cell[i] < minY ? cell[i] : minY;
    count += y_;
    total += y_*x_;
    sosq += x_*x_*y_;
}

string Histogram::printH() {
    assert(count != 0 && "Histogram::printH: Error printing a Histogram");

    double m = getMean();
    double v = getVariance();

    ostringstream out;
    out << name << "\n";
    out << "procesed = " << count << "\n";
    out << "mean = " << m << "\tvariance = " << v << "\tsd = " << sqrt(v) << "\n";
    out << "minX = " << minX << "\tmaxX = " << maxX << "\n";
    out << "minY = " << minY << "\tmaxY = " << maxY << "\n\n";

    for (int i = 0; i < cantCell; i++){
        if (i == 0) 
            out << "< " << setw(largoDouble-2) << lower;
        else if (i < cantCell - 1)
            out << setw(largoDouble) << lower + width * (i - 1);
        else out << "> " << setw(largoDouble-2) << upper;
        out << "  ";
        int cantAster = (int)floor((cell[i]/maxY)* (double)largoAster);
        for (int k = 0; k < cantAster; k++) 
            out << '*';
        for (int k = 0; k < largoAster - cantAster; k++) 
            out << ' ';
        out << "  ";
        out << setw(largoDouble) << cell[i];
        out << "\n";
    }
    return out.str();
}

string Histogram::printT() {
    assert(count != 0 && "Histogram::printT: Error printing a Histogram's table");

    double m = getMean();
    double v = getVariance();

    ostringstream out;
    out << name << "\n";
    out << "procesed\t" << count << "\n";
    out << "mean\t" << m << "\nvariance\t" << v << "\nsd\t" << sqrt(v) << "\n";
    out << "minX\t" << minX << "\nmaxX\t" << maxX << "\n";
    out << "minY\t" << minY << "\nmaxY\t" << maxY << "\n\n";

    for (int i = 0; i < cantCell; i++){
        if (i == 0) 
            out << "< " << setw(largoDouble-2) << lower;
        else if (i < cantCell - 1)
            out << setw(largoDouble) << lower + width * (i - 1);
        else out << "> " << setw(largoDouble-2) << upper;
        out << "\t";
        out << setw(largoDouble) << cell[i];
        out << "\n";
    }
    return out.str();
}

Histogram::~Histogram() {}

void Histogram::reset() {
    count = total = sosq = minX = maxX = minY = maxY = 0;
    for (int i = 0; i < cantCell; i++) 
        cell[i] = 0;
}

void Histogram::setModel(eosim::core::Model* model_) {
    assert(model == 0 && model_ != 0 && "Histogram::setModel: Error connecting a model to the Histogram");
    model = model_;
}

void Histogram::print() {
    cout << printH();
}

void Histogram::print(const char* path_) {
    ofstream out (path_);
    out << printH();
}

void Histogram::table() {
    cout << printT();
}

void Histogram::table(const char* path_) {
    ofstream out (path_);
    out << printT();
}

double Histogram::getMaxX() {
    assert(count != 0 && "Histogram::getMaxX: Error getting x's max value of a Histogram");
    return maxX;
}

double Histogram::getMinX() {
    assert(count != 0 && "Histogram::getMinX: Error getting x's min value of a Histogram");
    return minX;
}

double Histogram::getMaxY() {
    assert(count != 0 && "Histogram::getMaxY: Error getting y's max value of a Histogram");
    return maxY;
}

double Histogram::getMinY() {
    assert(count != 0 && "Histogram::getMinY: Error getting y's min value of a Histogram");
    return minY;
}


double Histogram::getVariance() {
    assert(count != 0 && "Histogram::getVariance: Error getting the variance of a Histogram");
    return (count > 1) ? (sosq-total*getMean())/(count-1): 0;
}

double Histogram::getMean() { 
    assert(count != 0 && "Histogram::getMean: Error getting mean value of a Histogram");
    return total / count;
}

string Histogram::getName() { 
    assert(count != 0 && "Histogram::getName: Error getting a Histogram's name");
    return name;
}

